
#include "comm.h"


Comm::Comm()
{ 
    hostname = "";
    username = "";
    password = "";
    setStatus(false);
}


int
Comm::login(string hostname_in, string username_in, string password_in)
{
    // Check if already logged in
    if (getStatus()) 
	{
	    logout();
	}

    hostname = hostname_in;
    username = username_in;
    password = password_in;
  
    LyStr client_name(PACKAGE);
    LyStr client_version(VERSION);
  
    // Create connection
    // The Connection-constructor only needs the string during the connection, 
    // so it's safe to deallocate "hostnames" memory. 

    conn = new Connection((char *) hostname.c_str(), "4894");
    if (conn->connection_failed())
	{
	    delete conn;
	    return 1; // Connection failed
	}
    session = new Session(conn, client_name, client_version);
  
    // Fetch user number
    vector<Conf_z_info> matches = 
	session->lookup_z_name(LyStr(username.c_str()), true, false);
    // Check is user exists
    if (matches.empty())
	{
	    delete session;
	    delete conn;
	    return 2; // Wrong username
	}
    usernumber = matches[0].conf_no();
    username = lystr2str(matches[0].conf_name());
    // Login
    LyStr password_lystr(password.c_str());
    if (session->login(usernumber, LyStr(password.c_str())) != st_ok)
	{
	    delete session;
	    delete conn;
	    return 3; // Wrong password
	}
    setStatus(true);  
    unread_confs = new Unread_confs(session, usernumber);  
    return 0;
};


void 
Comm::logout()
{ 
    // Check if already logged out
    if (getStatus()) {
	setStatus(false);
	// Logou
	session->logout();
	delete unread_confs;
	delete session;
	delete conn;
    }
}


vector< longstringpair >
Comm::getConferences(string str, bool pers, bool conf)
    // Fetch full conferencelist and/or username list
{
    vector< longstringpair > result;
    // Search
    vector<Conf_z_info> matches = 
	session->lookup_z_name(LyStr(str.c_str()), pers, conf);

    for (vector<Conf_z_info>::iterator iter = matches.begin();
	 iter != matches.end(); ++iter)
	{
	    longstringpair
		pair_temp((unsigned int) iter->conf_no(), (string) lystr2str(iter->conf_name()));
	    result.push_back(pair_temp);
	}
    return result;
}


bool
Comm::joinConference(unsigned int conference_id)
{
    return (session->add_member(conference_id, usernumber, 0, 0) == st_ok);
}


bool
Comm::leaveConference(unsigned int conference_id)
{
    return (session->sub_member(conference_id, usernumber) == KOM_NO_ERROR);
}


Article
Comm::getNextUnreadArticle()
{
    Article article;
    Conferences &  conferences = session->conferences();
    Texts & texts = session->texts();
    Text      ts;
    unsigned int number;
    unsigned long int tn;
    bool text_exist;
    vector<Micro_conf>  matches;

    do {
	// Loop until something can be returned
	if (unread_confs->next_prompt() == NEXT_CONF) {
	    // No more articles in this conference
	    article.setArticleId(0);
	    article.setSender(article.makePair(NEXT_CONF, 
					       "Slut p meddelanden i mte"));
	    return article;
	} 
	    
	if (unread_confs->next_prompt() == NO_MORE_TEXT) {
	    // No more articles on server
	    article.setArticleId(0);
	    article.setSender(article.makePair(NO_MORE_TEXT, 
					       "Slut p meddelanden p servern"));
	    return article;
	} 
	    
	tn = unread_confs->next_text(true);
	ts = texts[tn];
	text_exist = ts.exists();
	if (text_exist)
	    ts.mark_as_read();
    }
    while (!text_exist);
    
    // Sets reciever in article object
    for (int i = 0; i < ts.num_recipients(); ++i) {
	number = ts.recipient(i)->recipient();
	if (conferences[number].exists())
	    article.setReciever(
				article.makePair(number,
						 lystr2str(conferences[number].name())));
    }

    // Sets sender in article object
    if (conferences[ts.author()].exists()) {
	article.setSender(
			  article.makePair(ts.author(), 
					   lystr2str(conferences[ts.author()].name())));
    }
    else {
	article.setSender(
			  article.makePair(0, lystr2str("Oknd avsndare")));
    }

    article.setHeader(lystr2str(ts.subject().str));
    article.setText(lystr2str(ts.body().str));
    article.setTime(ts.creation_time());
    article.setArticleId(ts.text_no());

    // Sets "commentin" in article object
    for (int i = 0; i < ts.num_comment_in(); ++i)  {
	Text_no   comment;
	comment = ts.comment_in(i);
	if (texts[comment].exists()) {
	    if (conferences[texts[comment].author()].exists()) {
		article.setCommentIn(
				     article.makePair(comment,
						      lystr2str(conferences[texts[comment].author()].name())));
	    }
	}
    }

    // Sets "commentto" in article object
    for (int i = 0; i < ts.num_uplinks(); ++i) { 
	Text_no   parent; 
	parent = ts.uplink(i)->parent(); 
	if (texts[parent].exists()) {
	    if (conferences[texts[parent].author()].exists()) {
		article.setCommentTo( 
				     article.makePair(parent, 
						      lystr2str(conferences[texts[parent].author()].name()))); 
	    }
	}
    }
    
    // Sets footnote in article object
    for (int i = 0; i < ts.num_footnote_in(); ++i)  {
	Text_no   footnote;
	footnote = ts.footnote_in(i);
	if (texts[footnote].exists()) {
	    if (conferences[texts[footnote].author()].exists()) {
		article.setFootnote(
				    article.makePair(footnote,
						     lystr2str(conferences[texts[footnote].author()].name())));
	    }
	}
    }
    return article;
};


bool
Comm::setUnread(unsigned long int number)
{
    bool operation_ok;
    long int last_text_number, first_text_number, number_of_texts;
    Conferences &all_confs = unread_confs->session()->conferences();

    first_text_number =  all_confs[unread_confs->current_conf()].first_local_no();
    number_of_texts = all_confs[unread_confs->current_conf()].no_of_texts();
    // Calc first unread article number
    last_text_number = first_text_number + number_of_texts - number - 1;
    
    if (last_text_number < first_text_number) {
	// Sets all articles as unread
	operation_ok = 
	    (unread_confs->set_last_read(
					 unread_confs->current_conf(), 
					 first_text_number - 1) != KOM_INDETERMINATE);
	return false;
    }
    else {
	// Sets correct number of unread
	operation_ok=
	    (unread_confs->set_last_read(
					 unread_confs->current_conf(), 
					 last_text_number) != KOM_INDETERMINATE);
	return true;
    }
}


long int
Comm::getUnread()
{
    return unread_confs->estimated_unread_texts_in_current();
}


long int
Comm::getNumberOfArticles()
{
    Conferences &all_confs = unread_confs->session()->conferences();
    return all_confs[unread_confs->current_conf()].no_of_texts();
}


bool
Comm::sendArticle(Article article)
{
    bool sending_ok = false;
    string article_string = article.getHeader() + "\n" + article.getText();
    const LyStr article_text(article_string.c_str());

    // Reciepient
    vector <Recipient> reciever;
    vector < longstringpair > reciever_in = article.getReciever();
    vector < long unsigned int > comment_to, footnote;
    for (unsigned int i= 0 ; i < reciever_in.size(); i++) {
	reciever.push_back(Recipient((reciever_in[i]).first, false));
    }
    // Comment to
    vector < longstringpair > comment_to_in = article.getCommentTo();
    for (unsigned int i= 0 ; i < comment_to_in.size(); i++) {
	comment_to.push_back(comment_to_in[i].first);
    }
    // Footnote to 
    vector < longstringpair >footnote_in = article.getFootnote();
    for (unsigned int i= 0 ; i < footnote_in.size(); i++) {
	footnote.push_back(footnote_in[i].first);
    }
  
    create_text_question memberships(conn, 
				     article_text,
				     reciever,
				     comment_to,
				     footnote);
  
    if (memberships.receive() == st_ok) {
	sending_ok = true;
    }  
    else {
	sending_ok = false;
    }

    return sending_ok;
};


vector< longstringpair >
Comm::getMemberConferences()
{
    Conferences & conferences = session->conferences();
    vector< longstringpair > result;
    get_membership_list_question memberships(conn, usernumber, 0, 65535);
    if (memberships.receive() == st_ok) {
	vector<Membership> result_mem = memberships.result();
	for (vector<Membership>::iterator iter = result_mem.begin();
	     iter < result_mem.end(); ++iter) {
	    longstringpair
		pair_temp((unsigned int) iter->conf_no(),
			  lystr2str(conferences[iter->conf_no()].name()));
	    result.push_back(pair_temp);
	}  
    }
    else {
	// Return error article
	longstringpair pair_temp(0, "Inget mte");
	result.push_back(pair_temp);
    }

    return result;
}


bool Comm::changeConference(unsigned int conference_id)
{
    return (unread_confs->goto_conf(conference_id) == st_ok);
}


longstringpair
Comm::getNextConference()
{
    unsigned int next_conf = unread_confs->query_next_conf();
    if (next_conf == 0)  {
	// No more unread on server
	longstringpair
	    next_conference(0 , "Inget mte");
	return next_conference;
    } 
    else {
	longstringpair
	    next_conference(next_conf, 
			    lystr2str(session->conferences()[next_conf].name()));
	return next_conference;
    }
    // Dummy
    longstringpair next_conference(0, "Ngonting gick fel");
    return next_conference;
}


struct std::tm
Comm::getTime()
{
    return session->get_time();
};


string
Comm::getTimeString()
{
    //setlocale(LC_TIME, NULL);  // Now in main.cc
    char cstr[80];
    // %c r "The preferred date and time representation"
    strftime(cstr, 80, "%c", &(session->get_time()));
    return cstr;
};


unsigned long int 
Comm::getUserId()
{
    return usernumber;
}


void
Comm::setStatus(bool status_in)
{
    status = status_in;
}


bool
Comm::getStatus()
{
    return status;
}


string
Comm::lystr2str(const LyStr &lystr)
    // Convert LyStr to string
{
    // Create C-string from LyStr
    char *cstr;
    cstr = lystr.malloc_copy(0);
    // Create C++ string from C-string
    string str;
    str = cstr;
    delete [] cstr;
    return str;
}
