#include "config.h"
#include "joindialog.h"
#include "mainwindow.h"
#include "comm.h"

extern GtkWidget *topwindow;
extern Comm comm;

#include "controljoindialog.cc"

ControlJoinDialog* JoinDialog::controlObject=0;
GtkWidget* JoinDialog::window=NULL;
GtkWidget* JoinDialog::conf_list=NULL;
GtkWidget* JoinDialog::search1=NULL;
// -1 means nothing is selected
long int JoinDialog::selected_row=-1;


JoinDialog::JoinDialog(ControlJoinDialog *creator)
{
    controlObject = creator;
}


JoinDialog::~JoinDialog()
{

}


void
JoinDialog::ok_clicked(GtkWidget *widget,GdkEvent *event,gpointer data)
{
    // Make sure something is selected
    if (selected_row!=-1) 
	{
	    // Fetch data from the marked row
	    gchar *conference_number,*conference_name;
	    longstringpair res_pair;
	    
	    gtk_clist_get_text(GTK_CLIST(conf_list), selected_row, 0,
			       &conference_number);
      
	    // Convert string to unsigned int
	    res_pair.first = (unsigned int) atol(conference_number);
	    
	    // Fetch conference name
	    gtk_clist_get_text(GTK_CLIST(conf_list), selected_row, 1,
			       &conference_name);
	    
	    res_pair.second=conference_name;
	    	    
	    // Call controlclass 
	    controlObject->join(res_pair);
	}
    else
	; //NOP
    
}


void
JoinDialog::search_clicked(GtkWidget *widget,gpointer data)
{
    selected_row=-1;
  
    // Fetch searchstring fron entry 
    string query_string=(string)gtk_entry_get_text(GTK_ENTRY(search1)); 
    // Get conferences from the server
    vector< longstringpair > conference_list=
	controlObject->getMatchingConferences(query_string);
    // Clear conference list
    gtk_clist_clear(GTK_CLIST(conf_list));
    gtk_clist_freeze(GTK_CLIST(conf_list));
  
    // Fill the list with data
    int length_of_vector=conference_list.size();
    gchar *row_text[2];
    char tmp[80]; 
  
    for(int i=0;i<length_of_vector;i++ )
	{
	    // Conference name
	    row_text[1]= (gchar *) (conference_list[i].second).c_str();
      
	    // Conference number
	    sprintf(tmp, "%u", conference_list[i].first); 
	    row_text[0]= (gchar *) tmp;

	    gtk_clist_append(GTK_CLIST(conf_list),row_text);
      
	}
    gtk_clist_thaw(GTK_CLIST(conf_list));
}


void
JoinDialog::selection_made(GtkWidget      *clist,
			   gint            row,
			   gint            column,
			   GdkEventButton *event,
			   gpointer        data )
{
    selected_row=row;
}


void
JoinDialog::hide()
{
    gtk_widget_destroy(window);

}


void
JoinDialog::show()
{
    selected_row=-1;
    window = gnome_dialog_new(
			      _("Join conference"),
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);
    gtk_window_set_default_size(GTK_WINDOW(window),500,500);
  
    gtk_window_set_policy(GTK_WINDOW(window),FALSE,TRUE,TRUE);  
  
    gtk_window_set_modal(GTK_WINDOW(window),TRUE);
  
    label=gtk_label_new
	(_("Write the first part of the conference you are looking for and press search"));
  
    searchbutton=gnome_stock_or_ordinary_button(_("   Search   "));
    hbox=gtk_hbox_new(FALSE,3); 
    search1=gtk_entry_new();
  
    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (window)->vbox),
			label, FALSE,FALSE, 0);

    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (window)->vbox),
			hbox, FALSE,FALSE, 0);

    gtk_box_pack_start (GTK_BOX (hbox),
			search1, TRUE,TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox),
			searchbutton, FALSE,FALSE, 0);

    gtk_widget_show(hbox);
    gtk_widget_show(search1);
    gtk_widget_show(searchbutton);
    gtk_widget_show(label);

    // A scrolled window
    scroll_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_win),
				    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  
    gtk_box_pack_start(GTK_BOX (GNOME_DIALOG (window)->vbox), 
		       scroll_win, TRUE, TRUE, 0);
    gtk_widget_show (scroll_win);
    
    // Add titles
    gchar *titles[2] = { 
	N_("Conference number dummy"), 
	N_("Available conferences dummy") 
    };
    titles[0] = _("Conference number");
    titles[1] = _("Available conferences");
    
    // Create new list
    conf_list = gtk_clist_new_with_titles(2, titles);
  
    // Only one selection is allowed
    gtk_clist_set_selection_mode(GTK_CLIST(conf_list), GTK_SELECTION_SINGLE);
  
    // Insensitive column titles
    gtk_clist_column_titles_passive(GTK_CLIST(conf_list));
  
    // Column 0 invisible
    gtk_clist_set_column_visibility(GTK_CLIST(conf_list), 0, FALSE);
  
    gtk_clist_set_column_auto_resize(GTK_CLIST (conf_list),
				     1,
				     TRUE);
    
    gtk_container_add(GTK_CONTAINER(scroll_win),conf_list);
  
    gtk_widget_show(conf_list);
  
    // OK button as default
    gnome_dialog_set_default (GNOME_DIALOG(window),0);

    // Connect OK button
    gnome_dialog_button_connect
	(GNOME_DIALOG( window),0,
	 GTK_SIGNAL_FUNC(JoinDialog::ok_clicked)
	 ,NULL); 
    	 
    // Connect Cancel button
    gnome_dialog_button_connect_object
	(GNOME_DIALOG(window),1,
	 GTK_SIGNAL_FUNC(gtk_widget_destroy),
	 GTK_OBJECT(window));


    // Connect selection 
    gtk_signal_connect(GTK_OBJECT(conf_list), "select_row",
		       GTK_SIGNAL_FUNC(JoinDialog::selection_made),
		       NULL);

    // Connect search
    gtk_signal_connect(GTK_OBJECT(searchbutton), "clicked",
		       GTK_SIGNAL_FUNC(JoinDialog::search_clicked),
		       NULL);

    // Connect <RETURN> in search entry
    gtk_signal_connect(GTK_OBJECT(search1), "activate",
		       GTK_SIGNAL_FUNC(JoinDialog::search_clicked),
		       NULL);
  
    gnome_dialog_set_parent(GNOME_DIALOG(window),GTK_WINDOW(topwindow));

    gtk_widget_grab_focus(GTK_WIDGET(search1));
  
    gtk_widget_show(window);
  
    // Autosort
    gtk_clist_set_sort_column(GTK_CLIST(conf_list),1); 
    gtk_clist_set_sort_type(GTK_CLIST(conf_list),GTK_SORT_ASCENDING);
    gtk_clist_set_auto_sort(GTK_CLIST(conf_list),TRUE);
  
}


