#include <string>  

#include "mainwindow.h"
#include "aboutdialog.h"
#include "logoutdialog.h"
#include "comm.h"
#include "errordialog.h"
#include "exitdialog.h"
#include "logindialog.h"
#include "setunreaddialog.h"
#include "leavedialog.h"
#include "joindialog.h" 
#include "conferenceset.h" 
#include "comm.h" 
#include "writedialog.h" 

extern ControlMainWindow mainwin;
extern GtkWidget *topwindow;
extern ConferenceSet *conferenceSet; 
extern Comm comm; 
extern Article currentArticle;


// Menu definition
static GnomeUIInfo file_menu [] = {

    { GNOME_APP_UI_ITEM, 
      N_("_Login..."), 
      N_("Login to server"), 
      MainWindow::login_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'l', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("Log_out"), 
      N_("Logout from server"), 
      MainWindow::logout_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'u', GDK_CONTROL_MASK, NULL 
    },

    GNOMEUIINFO_SEPARATOR,

    GNOMEUIINFO_MENU_EXIT_ITEM(MainWindow::exit_cb, NULL),

    GNOMEUIINFO_END
};


static GnomeUIInfo conference_menu [] = {

    { GNOME_APP_UI_ITEM, 
      N_("_Join conference..."), 
      N_("Join a conference"), 
      MainWindow::join_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'm', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("Leave conference..."), 
      N_("Leave a conference"), 
      MainWindow::leave_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      't', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("_Next conference"), 
      N_("Go to next conference"), 
      MainWindow::next_conference_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'n', GDK_CONTROL_MASK, NULL 
    },
    GNOMEUIINFO_END
};


static GnomeUIInfo article_menu [] = {
    { GNOME_APP_UI_ITEM, 
      N_("_Read next unread"), 
      N_("Read next unread article"),
      MainWindow::next_unread_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      ' ', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("_Set unread"), 
      N_("Set number of unread"), 
      MainWindow::set_unread_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'e', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("_Write article"), 
      N_("Write a new article"), 
      MainWindow::write_article_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'i', GDK_CONTROL_MASK, NULL 
    },
    { GNOME_APP_UI_ITEM, 
      N_("_Comment article"), 
      N_("Write a comment to the last article"), 
      MainWindow::write_reply_cb, NULL, NULL, 
      GNOME_APP_PIXMAP_NONE, 0, 
      'k', GDK_CONTROL_MASK, NULL 
    },

    GNOMEUIINFO_END
};


static GnomeUIInfo help_menu [] = {

    GNOMEUIINFO_MENU_ABOUT_ITEM(MainWindow::about_cb, NULL), 
    
    GNOMEUIINFO_END
};

    
static GnomeUIInfo menu [] = {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_SUBTREE("_Conference", conference_menu),
    GNOMEUIINFO_SUBTREE("_Article", article_menu),
    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
};


#include "controlmainwindow.cc"


ControlMainWindow* MainWindow::controlObject;
GtkWidget* MainWindow::conf_list;


MainWindow::MainWindow(ControlMainWindow *creator)
{
    MainWindow::controlObject = creator;
}


MainWindow::~MainWindow()
{

}


void
MainWindow::dim_for_loggedin()
{
    // Deactivate these
    // Login
    gtk_widget_set_sensitive (file_menu[0].widget, FALSE);

    // Activate these
    // Logout
    gtk_widget_set_sensitive (file_menu[1].widget, TRUE);
    // All in conference menu
    gtk_widget_set_sensitive (conference_menu[0].widget, TRUE);
    gtk_widget_set_sensitive (conference_menu[1].widget, TRUE);
    gtk_widget_set_sensitive (conference_menu[2].widget, TRUE);
    // All in article menu
    gtk_widget_set_sensitive (article_menu[0].widget, TRUE);
    gtk_widget_set_sensitive (article_menu[1].widget, TRUE);
    gtk_widget_set_sensitive (article_menu[2].widget, TRUE);
    gtk_widget_set_sensitive (article_menu[3].widget, TRUE);

}


void
MainWindow::dim_for_loggedout()
{
    // Deactivate these
    // Logout
    gtk_widget_set_sensitive (file_menu[1].widget, FALSE);

    // All in conference menu
    gtk_widget_set_sensitive (conference_menu[0].widget, FALSE);
    gtk_widget_set_sensitive (conference_menu[1].widget, FALSE);
    gtk_widget_set_sensitive (conference_menu[2].widget, FALSE);
    // All in article menu
    gtk_widget_set_sensitive (article_menu[0].widget, FALSE);
    gtk_widget_set_sensitive (article_menu[1].widget, FALSE);
    gtk_widget_set_sensitive (article_menu[2].widget, FALSE);
    gtk_widget_set_sensitive (article_menu[3].widget, FALSE);

    // Activate these
    // Login
    gtk_widget_set_sensitive (file_menu[0].widget, TRUE);

}


void 
MainWindow::add_string_to_textwindow(string text)
{
    gtk_text_freeze (GTK_TEXT (textwindow));
    gtk_text_insert (GTK_TEXT (textwindow), NULL, 
		     &textwindow->style->black, NULL, text.c_str(), -1);
    gtk_text_thaw (GTK_TEXT (textwindow));

    // Scroll to the bottom
    GtkAdjustment *vadj;
    vadj = 
	gtk_scrolled_window_get_vadjustment(
					    GTK_SCROLLED_WINDOW(scrolled_window_right));
    
    gtk_adjustment_set_value(vadj, vadj->upper);
}


void 
MainWindow::color_add(const char *addtext, GdkColor *fore)
{
    gtk_text_insert (GTK_TEXT (textwindow), NULL, fore, NULL, 
		     addtext, -1);

}


void 
MainWindow::add_to_textwindow(Article article)

{
    GdkColor black_color = { 0, 0x0000, 0x0000, 0x0000 };
    GdkColor white_color = { 0, 0xFFFF, 0xFFFF, 0xFFFF };
    GdkColor red_color = { 0, 0xFFFF, 0x0000, 0x0000 };
    GdkColor green_color = { 0, 0x0000, 0xFFFF, 0x0000 };
    GdkColor blue_color = { 0, 0x0000, 0x0000, 0xFFFF };
    GdkColor cyan_color = { 0, 0x0000, 0xFFFF, 0xFFFF };
    GdkColor magenta_color = { 0, 0xFFFF, 0x0000, 0xFFFF };
    GdkColor yellow_color = { 0, 0xFFFF, 0xFFFF, 0x0000 };
    
    char tmp[80];
    string nl="\n", str, line;
    gtk_text_freeze (GTK_TEXT (textwindow));

    // Policy: All objekt should add space and a line break (when necessary)

    // Article number
    sprintf(tmp, "%lu ", article.getArticleId());
    color_add(tmp, &blue_color);

    // Time
    str = article.getTimeString() + " ";
    color_add(str.c_str(), &black_color);

    // Sender, last in row
    str = article.getSender().second + nl;
    color_add(str.c_str(), &blue_color);

    // Comment to
    vector < longstringpair > comment_to;
    comment_to = article.getCommentTo();
    for (unsigned int i = 0; i < comment_to.size(); i++)
 	{
	    color_add(_("Comment to article "), &black_color);
	    sprintf(tmp, "%lu ", (comment_to[i]).first);
	    color_add(tmp, &blue_color);
	    color_add("by ", &black_color);
	    str = (comment_to[i]).second + nl;
	    color_add(str.c_str(), &blue_color);
	}   

    // Recipient
    vector < longstringpair > receivers;
    receivers = article.getReciever();
    for (unsigned int i = 0; i < receivers.size(); i++)
 	{
	    color_add(_("Recipient: "), &black_color);
	    str = receivers[i].second + nl;
	    color_add(str.c_str(), &blue_color);
	}   

    // Subject
    str = _("Subject: ");
    color_add(str.c_str(), &black_color);
    str = article.getHeader() + nl;
    color_add(str.c_str(), &black_color);

    // Separation line
    str = "";
    for (int i = 1; i < 60; i++) { str += "-"; };
    str += nl;
    color_add(str.c_str(), &black_color);

    // Article text
    str = article.getText() + nl;
    color_add(str.c_str(), &black_color);

    // Foot
    color_add("(", &black_color);
    sprintf(tmp, "%lu", article.getArticleId());
    color_add(tmp, &blue_color);
    color_add(") ", &black_color);
    longstringpair sender;
    sender = article.getSender();
    str = "/" + sender.second + "/" + nl;
    color_add(str.c_str(), &blue_color);

    // Comment in
    vector < longstringpair > comment_in;
    comment_in = article.getCommentIn();
    for (unsigned int i = 0; i < comment_in.size(); i++)
 	{
	    color_add(_("Comment in article "), &black_color);
	    sprintf(tmp, "%lu ", (comment_in[i]).first);
	    color_add(tmp, &blue_color);
	    color_add("by ", &black_color);
	    str = (comment_in[i]).second + nl;
	    color_add(str.c_str(), &blue_color);
	}   

    // Ending line
    str = "";
    for (int i = 1; i < 60; i++) { str += "-"; };
    str += nl + nl;
    color_add(str.c_str(), &black_color);
    
    gtk_text_thaw (GTK_TEXT (textwindow));

    // Scroll to the bottom
    GtkAdjustment *vadj;
    vadj = 
	gtk_scrolled_window_get_vadjustment(
					    GTK_SCROLLED_WINDOW(scrolled_window_right));
     
    gtk_adjustment_set_value(vadj, vadj->upper);
}


GtkWidget*
MainWindow::conferencelist_create()
{
    // Column 0 is for future pixmap
    // Column 1 (the only column visible) is for the conference name
    // Column 2 contains the conference number
    gchar *titles[3] = { 
	N_("Icon dummy"), 
	N_("Conference dummy"), 
	N_("Conference number dummy") 
    };
    titles[0] = _("Icon");
    titles[1] = _("Conference");
    titles[2] = _("Conference number");
    
    conf_list = gtk_clist_new_with_titles(3, titles);
    gtk_clist_set_selection_mode(GTK_CLIST(conf_list), GTK_SELECTION_BROWSE);

    //    gtk_widget_remove_accelerator(GTK_WIDGET(conf_list), GNOME_APP(topwindow)->accel_group,
    //				  GDK_space, 0);

    // Callback
    gtk_signal_connect(GTK_OBJECT(conf_list), "select_row",
                       GTK_SIGNAL_FUNC(MainWindow::conference_clicked),
                       NULL);
    
    gtk_clist_column_titles_passive(GTK_CLIST(conf_list));

    // Column 0 and 2 should be invisible
    gtk_clist_set_column_visibility(GTK_CLIST(conf_list), 0, FALSE);
    gtk_clist_set_column_visibility(GTK_CLIST(conf_list), 2, FALSE);

    gtk_clist_set_column_width(GTK_CLIST(conf_list), 1, 60);
    gtk_clist_set_column_auto_resize(GTK_CLIST(conf_list), 1, TRUE);

			       
    return conf_list;

}


void 
MainWindow::conferencelist_add(longstringpair apair)
{
    char tmp[80];
    gchar *row[3];

    sprintf(tmp, "%d", apair.first);
    
    row[0] = "";
    row[1] = (gchar *) apair.second.c_str();
    row[2] = tmp;

    gtk_clist_append(GTK_CLIST(conf_list), row);
}


void 
MainWindow::conferencelist_del(unsigned int conf_num)
{
    guint32 i;
    gchar *text;

    for (i = 0; i < 65000; i++) {
	gtk_clist_get_text(GTK_CLIST(conf_list), i, 2, &text);    
	if ( (unsigned int) atol(text) == conf_num )
	    break;
    }

    assert(i < 65000);
    
    gtk_clist_remove(GTK_CLIST(conf_list), i);
}


void 
MainWindow::conferencelist_select(unsigned int conf_num)
{
    guint32 i;
    gchar *text;

    for (i = 0; i < 65000; i++) {
	gtk_clist_get_text(GTK_CLIST(conf_list), i, 2, &text);    
	if ( (unsigned int) atol(text) == conf_num )
	    break;
    }

    assert(i < 65000);
    
    gtk_clist_select_row(GTK_CLIST(conf_list), i, 1);
}


void 
MainWindow::conferencelist_clear()
{

    gtk_clist_clear(GTK_CLIST(conf_list));
}


void 
MainWindow::conference_clicked(GtkWidget      *clist,
			       gint            row,
			       gint            column,
			       GdkEventButton *event,
			       gpointer        data)
{
    gchar *conf_name, *conf_num;

    gtk_clist_get_text(GTK_CLIST(conf_list), row, 1, &conf_name);
    gtk_clist_get_text(GTK_CLIST(conf_list), row, 2, &conf_num);

    longstringpair apair;

    apair.first = (unsigned int) atol(conf_num);
    apair.second = conf_name;

    controlObject->conferenceClicked(apair);
}


void
MainWindow::show()
{

    GtkWidget *hpaned;
    GtkWidget *left;
    
    // Create main window
    topwindow = gnome_app_new(PACKAGE, PACKAGE);
    gtk_window_set_default_size(GTK_WINDOW (topwindow), 700, 500);
    install_menus_and_toolbar(topwindow);

    dim_for_loggedout();
    gtk_signal_connect (GTK_OBJECT (topwindow), "delete_event",                      
			GTK_SIGNAL_FUNC (MainWindow::exit_cb), NULL);

    // Make two panes
    hpaned = gtk_hpaned_new();
    gnome_app_set_contents (GNOME_APP(topwindow), hpaned);
    gtk_paned_set_handle_size (GTK_PANED(hpaned),
                               10);
    gtk_paned_set_gutter_size (GTK_PANED(hpaned), 10);
    gtk_paned_set_position (GTK_PANED(hpaned), 150);

    gtk_widget_show (hpaned);

    // Left pane
    // *********
    GtkWidget *scrolled_window_left;


    left = conferencelist_create();
    scrolled_window_left = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window_left),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_add(GTK_CONTAINER(scrolled_window_left), left);

    gtk_paned_add1 (GTK_PANED(hpaned), scrolled_window_left);

    gtk_widget_show(left);
    gtk_widget_show(scrolled_window_left);

    // Right pane
    // **********
    textwindow = gtk_text_new (NULL, NULL);
    scrolled_window_right = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window_right),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_add(GTK_CONTAINER(scrolled_window_right), textwindow);

    gtk_paned_add2 (GTK_PANED(hpaned), scrolled_window_right);

    gtk_widget_show(textwindow);
    gtk_widget_show(scrolled_window_right);

    gtk_widget_show(hpaned);

    // Show main window
    gtk_widget_show (topwindow);

}


void
MainWindow::install_menus_and_toolbar(GtkWidget* app)
{
    // Create menus
    gnome_app_create_menus (GNOME_APP(app), menu);

    add_elisp_accelerators(app);
    // Space requires extra attention. Listen globally.
    gtk_signal_connect (GTK_OBJECT (topwindow),                                 
			"key_press_event", 
                        GTK_SIGNAL_FUNC (key_press_cb), NULL); 
    
    // Create status bar
    appbar = 
	GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER));
    gnome_app_set_statusbar(GNOME_APP(app), GTK_WIDGET(appbar));
    
    // Installer menu hints
    gnome_app_install_menu_hints(GNOME_APP(app), menu);
}


void 
MainWindow::appbarStatus(string text)
{
    gnome_appbar_set_default (appbar, text.c_str());
}


void 
MainWindow::add_elisp_accelerators(GtkWidget* app)
{
    /* Logout */
    gtk_widget_add_accelerator (file_menu[1].widget, "activate", GNOME_APP(app)->accel_group,
				GDK_q, 0, (GtkAccelFlags) 0);
    /* Join conference */
    gtk_widget_add_accelerator (conference_menu[0].widget, "activate", GNOME_APP(app)->accel_group,
				GDK_m, 0, (GtkAccelFlags) 0);
    /* Read next unread */
    //   gtk_widget_add_accelerator (article_menu[0].widget, "activate", GNOME_APP(app)->accel_group,
    //  				GDK_space, 0, (GtkAccelFlags) 0);
    
    /* Set unread */
    gtk_widget_add_accelerator (article_menu[1].widget, "activate", GNOME_APP(app)->accel_group,
				GDK_e, 0, (GtkAccelFlags) 0);
    /* Write article */
    gtk_widget_add_accelerator (article_menu[2].widget, "activate", GNOME_APP(app)->accel_group,
				GDK_i, 0, (GtkAccelFlags) 0);
    /* Comment article */
    gtk_widget_add_accelerator (article_menu[3].widget, "activate", GNOME_APP(app)->accel_group,
				GDK_k, 0, (GtkAccelFlags) 0);
}


void                                                                            
MainWindow::key_press_cb(GtkWidget *widget, GdkEventKey *event)                 
{                                                                               
    if (comm.getStatus()) {                                                                       
	// Only if logged in
	switch (event->keyval) {                                                               
	case ' ':                                                       
	    controlObject->next_unread();
	    break;                                
	}
    }
}


void 
MainWindow::login_cb(GtkWidget* widget, gpointer data)
{
    controlObject->login();
}


void 
MainWindow::logout_cb(GtkWidget* widget, gpointer data)
{
    controlObject->logout();
}


void 
MainWindow::exit_cb(GtkWidget* widget, gpointer data)
{
    controlObject->exit();
}

 
void 
MainWindow::leave_cb(GtkWidget* widget, gpointer data)
{
    controlObject->leave();
}


void 
MainWindow::join_cb(GtkWidget* widget, gpointer data)
{
    controlObject->join();
}


void 
MainWindow::next_conference_cb(GtkWidget* widget, gpointer data)
{
    controlObject->next_conference();
}


void 
MainWindow::next_unread_cb(GtkWidget* widget, gpointer data)
{
    controlObject->next_unread();
}


void 
MainWindow::set_unread_cb(GtkWidget* widget, gpointer data)
{
    controlObject->set_unread();
}


void 
MainWindow::write_article_cb(GtkWidget* widget, gpointer data)
{
    controlObject->write_article();
}


void 
MainWindow::write_reply_cb(GtkWidget* widget, gpointer data)
{
    controlObject->write_reply();
}


void 
MainWindow::about_cb(GtkWidget* widget, gpointer data)
{
    controlObject->about();
}









